searchData={"items":[{"type":"module","title":"Errata","doc":"Errata is an Elixir library that promotes a consistent and structured approach to\nerror handling.\n\nErrata provides support for defining custom structured error types, which can\neither be returned as error values or raised as exceptions.\n\nErrata errors are named, structured types that represent error conditions in an\napplication or library. Being named types means that errors have a unique and\nmeaningful name within a particular context. Being structured types means that\nerrors have a well-defined, consistent structure identifying the nature of the\nerror, and can also have arbitrary contextual information attached to them for\nlogging or debugging purposes.\n\nErrata errors fall into one of three general classifications:\n\n- _Domain Errors_ represent error conditions within a problem domain or bounded\n  context. These are business process violations or other errors in the problem\n  domain, and therefore domain errors should be included as part of the\n  [Ubiquitous Language](https://martinfowler.com/bliki/UbiquitousLanguage.html)\n  of the domain.\n- _Infrastructure Errors_ represent errors that can occur at an infrastructure\n  level but which are not part of the problem domain. Infrastructure errors\n  include such things as network timeouts, database connection failures,\n  filesystem errors, etc.\n- _General Errors_ represent errors that do not fall into either of the above\n  categories. General errors can include errors that emanate from library code\n  (as opposed to application code) or any other sort of error condition for\n  which there is no need to distinguish based on classification.","ref":"Errata.html"},{"type":"module","title":"Defining custom error types - Errata","doc":"Errata makes it easy to define custom error types for applications or libraries.\nThe following examples demonstrate how to define domain errors, infrastructure\nerrors, and general errors.\n\n```elixir\ndefmodule MyApp.SomeContext.MyDomainError do\n  # Define a custom domain error in some context.\n  use Errata.DomainError\nend\n\ndefmodule MyApp.SomeContext.MyInfrastructureError do\n  # Define a custom infrastructure error in some context.\n  use Errata.InfrastructureError\nend\n\ndefmodule MyApp.SomeContext.MyError do\n  # Define a custom error in some context.\n  use Errata.Error\nend\n```\n\nEach of these custom error types will define an exception struct that conforms\nto the `t:Errata.error/0` type and will implement the `Errata.Error` behaviour.\nAdditionally, implementations for the `String.Chars` protocol and the\n`Jason.Encoder` protocol are provided so that these errors can be converted to\nstring form or encoded as JSON automatically.","ref":"Errata.html#module-defining-custom-error-types"},{"type":"module","title":"Raising Errata errors as exceptions - Errata","doc":"Since Errata errors are just regular Elixir exceptions with a well-defined\nstructure, they can be raised just like any other type of exception.\n\n```elixir\nraise MyApp.SomeContext.MyDomainError, reason: :invalid_data, context: %{foo: \"bar\"}\n```","ref":"Errata.html#module-raising-errata-errors-as-exceptions"},{"type":"module","title":"Using Errata errors as return values - Errata","doc":"Errata errors can be created by using the `new/1` or `create/1` functions for\nthe error type. Once created, they can be used as return values from functions,\npreferably wrapped in an error tuple. This approach allows for creating errors\nwith full contextual details at the site of the error, while separating the\nhandling of errors by letting them propagate up the call stack and logging\nor reporting the errors at the boundaries of the system.\n\nUsing the `new/1` function creates an error with the provided data, but does\nnot include any data in the `:env` field of the error struct:\n\n```elixir\niex> alias MyApp.SomeContext.MyError\niex> MyError.new(reason: :invalid_data, context: %{foo: \"bar\"})\n%MyError{reason: :invalid_data, context: %{foo: \"bar\"}, env: nil}\n```\n\nUsing the `create/1` macro, on the other hand, fills in the `:env` field of\nthe error struct with information from the current `__ENV__` struct and\nthe current stacktrace. To use the `create/1` macro, you must first\n`require/2` the error module:\n\n```elixir\niex> require MyApp.SomeContext.MyError, as: MyError\niex> error = MyError.create(reason: :invalid_data, context: %{foo: \"bar\"})\niex> error.reason == :invalid_data\ntrue\niex> error.context == %{foo: \"bar\"}\ntrue\niex> match?(%Errata.Env{stacktrace: stacktrace} when is_list(stacktrace), error.env)\ntrue\n```\n\nWhether `new/1` or `create/1` is used to create the error, it is preferable to\nwrap the error in a tuple when returning it as a value from functions:\n\n```elixir\n{:error, MyError.new(reason: :invalid_data)}\n{:error, MyError.create(reason: :invalid_data)}\n```","ref":"Errata.html#module-using-errata-errors-as-return-values"},{"type":"module","title":"Handling Errata errors - Errata","doc":"Since error types defined using Errata are standard Elixir exceptions, they can\nbe handled in the same way as any other exception. Specifically, they can be\nused in the `rescue` clause of a `try` block or function, and the\n`Kernel.is_exception/1` guard will always return `true` for them. Additionally,\nErrata provides custom guards for handling Errata error types specifically:\n\n- `Errata.is_error/1`\n- `Errata.is_domain_error/1`\n- `Errata.is_infrastructure_error/1`\n\nTo use these custom guards, `import` or `require` the `Errata` module.\n\nThe following example demonstrates handling Errata errors both as raised\nexceptions and as error values returned from functions.\n\n```elixir\ndefmodule MyApp.SomeContext do\n  # require the Errata module to use the custom guards\n  require Errata\n\n  def handle_errata_error_as_exception do\n    try do\n      function_that_raises_errata_error!()\n    rescue\n      e in [MyApp.SomeContext.MyDomainError] ->\n        # Errata errors can be rescued by their specific type\n        handle_my_domain_error(e)\n\n      e when Errata.is_error(e) ->\n        # Or they can be rescued using one of the custom guards defined in the\n        # Errata module\n        handle_errata_error(e)\n\n      e ->\n        # Regular exceptions may be handled separately if desired\n        handle_other_error(e)\n    end\n  end\n\n  def handle_errata_error_as_value do\n    case function_that_returns_errata_error_as_value() do\n      {:ok, result} ->\n        handle_ok_result(result)\n\n      {:error, %MyApp.SomeContext.MyDomainError{} = error} ->\n        # Errata errors can be pattern matched by their specific type\n        handle_my_domain_error(error)\n\n      {:error, error} when Errata.is_error(error) ->\n        # Or they can be identified using one of the custom guards defined in\n        # the Errata module\n        handle_errata_error(e)\n\n      {:error, reason} ->\n        # Other errors may be handled separately if desired\n        handle_other_error(reason)\n    end\n  end\nend\n```","ref":"Errata.html#module-handling-errata-errors"},{"type":"module","title":"Compared to traditional approaches to error handling - Errata","doc":"It is common practice in Elixir (and Erlang) to use error tuples of the form\n`{:error, reason}` as return values from functions to indicate an error\ncondition. However, the `reason` value that is used as the second element\nin the tuple is often a simple value such as an atom or (worse) a string, and\ndoes not typically include sufficient context for interpreting the error.\nWhile these simple `reason` values are often sufficient for human readers of\nthe code when viewed in context, they do not provide enough context to be\ninterpreted by code or when viewed as log messages or error reports, where\nthe context of where and when the error was originally detected and created\nis not readily apparent.\n\nIt is a less common but still widespread practice to raise exceptions for\nerrors, instead of or in addition to returning error values from functions.\nAlthough exceptions do include some contextual information (including, in\nparticular, a stacktrace), they lack a common, uniform structure that can be\nused for logging and error handling in general.\n\nErrata, on the other hand, defines a uniform structure that all errors share,\nand allows errors to be created with full contextual details, including\narbitrary context metadata. This full context is embedded into the error struct\nso that it propagates with the error, whether the error is raised as an\nexception or returned as an error value from a function. Errata errors are also\nJSON-encodable so that they can be easily published to external issue tracking\nsystems such as Sentry, for example.\n\nConsider the common pattern pattern of using a `with` expression with a series\nof function calls, each of which is expected to return a tuple either of the\nform `{:ok, result}` or `{:error, reason}`. If the error `reason` does not\ncontain sufficient contextual detail about the nature and cause of the error,\nthen the `with` expression is forced to handle all possible error values in an\n`else` clause, in order to log or report the error in a meaningful way.\n\nIf, instead, the error `reason` for each error is a structured type with full\ncontext, the `with` expression can omit the `else` clause altogether and allow\nthe error to propagate to callers. Since the error includes sufficient context\nit can be logged or reported to an issue tracking system at the boundaries,\nsuch as a Phoenix controller or a bounded context, without losing the context\nnecessary for interpreting or debugging the error.\n\nChris Keathley provides an in-depth discussion of this point in his blog post\n[Good and Bad Elixir](https://keathley.io/blog/good-and-bad-elixir.html), in\nthe section \"Avoid `else` in `with` blocks\".","ref":"Errata.html#module-compared-to-traditional-approaches-to-error-handling"},{"type":"macro","title":"Errata.is_domain_error/1","doc":"Returns `true` if `term` is an Errata domain error type; otherwise returns `false`.\n\nAllowed in guard tests.","ref":"Errata.html#is_domain_error/1"},{"type":"macro","title":"Errata.is_error/1","doc":"Returns `true` if `term` is any Errata error type; otherwise returns `false`.\n\nAllowed in guard tests.","ref":"Errata.html#is_error/1"},{"type":"macro","title":"Errata.is_infrastructure_error/1","doc":"Returns `true` if `term` is an Errata infrastructure error type; otherwise returns `false`.\n\nAllowed in guard tests.","ref":"Errata.html#is_infrastructure_error/1"},{"type":"type","title":"Errata.domain_error/0","doc":"Type to represent Errata domain errors.","ref":"Errata.html#t:domain_error/0"},{"type":"type","title":"Errata.error/0","doc":"Type to represent any kind of Errata error.\n\nErrata errors are `Exception` structs that have additional fields to contain extra contextual\ninformation, such as an error reason or details about the context in which the error occurred.","ref":"Errata.html#t:error/0"},{"type":"type","title":"Errata.error_kind/0","doc":"Type to represent the various kinds of Errata errors.","ref":"Errata.html#t:error_kind/0"},{"type":"type","title":"Errata.infrastructure_error/0","doc":"Type to represent Errata infrastructure errors.","ref":"Errata.html#t:infrastructure_error/0"},{"type":"module","title":"Errata.DomainError","doc":"Domain errors represent error conditions within a problem domain or bounded context.\n\nDomain errors are business process violations, dataa consistency errors,  or other errors in the\nproblem domain. Therefore, domain errors should have a meaningful name within a particular\ncontext, and the precise meaning of that name should be part of the Ubiquitous Language of the\ncontext.\n\nDomain errors can be defined by creating an Elixir module that uses the `Errata.DomainError`\nmodule. Error types defined in this way are `Errata.Error` types of kind `:domain`. As such,\nthey share the common structure of all Errata error types and support all of the callbacks\ndefined by the `Errata.Error` behaviour.\n\nSee the module docs for `Errata.Error` for more details.","ref":"Errata.DomainError.html"},{"type":"module","title":"Usage - Errata.DomainError","doc":"To define a new custom domain error type, `use/2` the `Errata.DomainError` module in your own\nerror module:\n\n    defmodule MyApp.SomeContext.SomeError do\n      use Errata.DomainError,\n        default_message: \"something isn't right in this context\"\n    end","ref":"Errata.DomainError.html#module-usage"},{"type":"type","title":"Errata.DomainError.t/0","doc":"Type to represent Errata domain errors.","ref":"Errata.DomainError.html#t:t/0"},{"type":"module","title":"Errata.Env","doc":"A struct that holds compile time environment information that is used for the `:env` field of\nErrata error types.\n\nThis struct is a subset of `Macro.Env` and includes the following fields:\n\n  * `context` - the context of the environment; it can be nil (default context), :guard\n    (inside a guard) or :match (inside a match)\n  * `context_modules` - a list of modules defined in the current context\n  * `file` - the current absolute file name as a binary\n  * `function` - a tuple as {atom, integer}, where the first element is the function name and\n    the second its arity; returns nil if not inside a function\n  * `line` - the current line as an integer\n  * `module` - the current module name\n  * `stacktrace` - the stacktrace for the current process at the time of creation","ref":"Errata.Env.html"},{"type":"function","title":"Errata.Env.new/2","doc":"Creates a new `Errata.Env` struct from the given `Macro.Env` struct.","ref":"Errata.Env.html#new/2"},{"type":"function","title":"Errata.Env.to_map/1","doc":"Converts the given `Errata.Env` struct to a plain, JSON-encodable map.","ref":"Errata.Env.html#to_map/1"},{"type":"type","title":"Errata.Env.env_map/0","doc":"Type to represent an `Errata.Env` struct as a plain, JSON-encodable map.","ref":"Errata.Env.html#t:env_map/0"},{"type":"type","title":"Errata.Env.t/0","doc":"Type to represent the `:env` field of error structs.\n\nThis struct is a subset of `Macro.Env`.","ref":"Errata.Env.html#t:t/0"},{"type":"behaviour","title":"Errata.Error","doc":"Support for creating custom error types, which can either be returned as error values or raised\nas exceptions.\n\nErrata errors can be defined by creating an Elixir module that uses the `Errata.Error`\nmodule. Error types defined in this way are Elixir `Exception` structs with the following keys:\n\n  * `message` - human readable string describing the nature of the error\n  * `reason` - an atom describing the reason for the error, which can be used for pattern\n    matching or classifying the error\n  * `context` - a map containing arbitrary contextual information or metadata about the error\n\nBecause these error types are defined with `defexception/1`, they can be raised as exceptions\nwith `raise/2`. However, because they implement the `Errata.Error` behaviour, it is also\npossible to create instances of these error structs using the generated implementations of\n`c:Errata.Error.new/1` or `c:Errata.Error.create/1` and use them as return values from\nfunctions, either directly or wrapped in an error tuple such as `{:error, my_error}`.\n\nError types defined with `Errata.Error` are of kind `:general` by default. See also\n`Errata.DomainError` and `Errata.InfrastructureError` for defining domain errors and\ninfrastructure errors, specifically.","ref":"Errata.Error.html"},{"type":"behaviour","title":"Usage - Errata.Error","doc":"To define a new custom error type, `use/2` the `Errata.Error` module in your own error module:\n\n    defmodule MyApp.SomeError do\n      use Errata.Error,\n        default_message: \"something isn't right\"\n    end\n\n> #### `use Errata.Error` {: .info}\n>\n> When you `use Errata.Error`, the `Errata.Error` module will define an exception struct with\n> `defexception/1` and will generate an implementation of the `Errata.Error` behaviour.\n\nThe following options may be provided to `use Errata.Error`:\n\n  * `:default_reason` - the default value to use for the `:reason` field if it is not provided\n  * `:default_message` - the default value to use for the `:message` field if it is not provided\n  * `:kind` - the \"kind\" of Errata error to create, one of `:domain`, `:infrastructure`, or\n    `:general` (which is the default)\n\n> ##","ref":"Errata.Error.html#module-usage"},{"type":"behaviour","title":"The `:kind` option {: .warning} - Errata.Error","doc":">\n> Although it is possible to define domain error types or infrastructure error types by using\n> `:domain` or `:infrastructure` as the `:kind` option, it is preferred to instead define these\n> types of errors with `use Errata.DomainError` or `use Errata.InfrastructureError`. This\n> approach is more explicit and allows for easier identification of domain errors and\n> infrastructure errors within an application.\n\nTo create instances of the error--to use as an error return value from a function, say--you can\nuse either `c:new/1` or `c:create/1`, passing params with extra information as desired. Note that\nif you use `c:create/1`, you must first `require` the error module, since this callback is\nimplemented as a macro. For example:\n\n    defmodule MyApp.SomeModule do\n      require MyApp.SomeError, as: SomeError\n\n      def some_function(arg) do\n        {:error, SomeError.create(reason: :helpful_tag, context: %{arbitrary: \"metadata\", arg: arg})}\n      end\n    end\n\nTo raise errors as exceptions, simply use `raise/2` passing extra params as the second argument\nif desired:\n\n    defmodule MyApp.SomeModule do\n      require MyApp.SomeError, as: SomeError\n\n      def some_function!(arg) do\n        raise SomeError reason: :helpful_tag, context: %{arbitrary: \"metadata\", arg: arg}\n      end\n    end","ref":"Errata.Error.html#module-the-kind-option-warning"},{"type":"macrocallback","title":"Errata.Error.create/0","doc":"Invoked to create a new instance of an error struct with default values and the current\n`__ENV__`.\n\nSee `c:create/1`.","ref":"Errata.Error.html#c:create/0"},{"type":"macrocallback","title":"Errata.Error.create/1","doc":"Invoked to create a new instance of an error struct with the given params and the current\n`__ENV__`.\n\nSince this is a  macro, the `__ENV__/0` special form is used to capture the `Macro.Env` struct\nfor the current environment and the public fields of this struct are placed in the exception\nstruct under the `:env` key. This provides access to information about the context in which the\nerror was created, such as the module, function, file, and line. See `t:env/0` for further\ndetails.\n\nNote that because this is a macro, callers must `require/2` the error module to be able to use it.","ref":"Errata.Error.html#c:create/1"},{"type":"callback","title":"Errata.Error.new/0","doc":"Invoked to create a new instance of an error struct with default values.\n\nSee `c:new/1`.","ref":"Errata.Error.html#c:new/0"},{"type":"callback","title":"Errata.Error.new/1","doc":"Invoked to create a new instance of an error struct with the given params.","ref":"Errata.Error.html#c:new/1"},{"type":"callback","title":"Errata.Error.to_map/1","doc":"Invoked to convert an error to a plain, JSON-encodable map.","ref":"Errata.Error.html#c:to_map/1"},{"type":"type","title":"Errata.Error.param/0","doc":"Type to represent allowable keys to use in params used for creating error structs.\n\nSee also `t:params/0`.","ref":"Errata.Error.html#t:param/0"},{"type":"type","title":"Errata.Error.params/0","doc":"Type to represent allowable values to be passes as params for creating error structs.\n\nThis effectively allows for using either a map or keyword list with allowable keys defined by\n`t:param/0`.","ref":"Errata.Error.html#t:params/0"},{"type":"type","title":"Errata.Error.t/0","doc":"Type to represent Errata error structs.\n\nError structs are `Exception` structs that have additional fields to contain extra contextual\ninformation, such as an error reason or details about the context in which the error occurred.","ref":"Errata.Error.html#t:t/0"},{"type":"module","title":"Errata.InfrastructureError","doc":"Infrastructure errors represent errors that can occur at an infrastructure level but which\nare not part of the problem domain.\n\nInfrastructure errors include such things as network timeouts, database connection failures,\nfilesystem errors, etc. Unlike domain errors, infrastructure errors are not part of the problem\ndomain and therefore are not typically part of the Ubiquitous Language of the domain.\n\nInfrastructure errors can be defined by creating an Elixir module that uses the\n`Errata.InfrastructureError` module. Error types defined in this way are `Errata.Error` types of\nkind `:infrastructure`. As such, they share the common structure of all Errata error types and\nsupport all of the callbacks defined by the `Errata.Error` behaviour.\n\nSee the module docs for `Errata.Error` for more details.","ref":"Errata.InfrastructureError.html"},{"type":"module","title":"Usage - Errata.InfrastructureError","doc":"To define a new custom infrastructure error type, `use/2` the `Errata.InfrastructureError`\nmodule in your own error module:\n\n    defmodule MyApp.SomeContext.SomeError do\n      use Errata.InfrastructureError,\n        default_message: \"something isn't right with the infrastructure\"\n    end","ref":"Errata.InfrastructureError.html#module-usage"},{"type":"type","title":"Errata.InfrastructureError.t/0","doc":"Type to represent Errata infrastructure errors.","ref":"Errata.InfrastructureError.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}